/****************************************************************************
 * NAME : Dulith Herath
 * FILE : main.cc
 * PROJECT: CS 541
 * SOURCE:   Provided as a handout by Jerzy W. Jaromczynk
 *           with additionals and midifications by Dulith Herath
 ****************************************************************************/

/* fixed for modern compilers by Dirk Jagdmann <doj@cubic.org> */

#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <string.h>

#include "rdparser.h"

/* lex associated functions */
extern  void InitLex(void);
extern int numlines;
/* parser associated externs */
extern int rdlines;

/* Flags for output from rdparser.cc */
int debug_flag = 0;

/* The program name */
char *program_name = NULL;

/****************************************************************************
 * FUNCTION_NAME : usage()
 * OUTPUT        : program parameters.
 * VALUE RETURNED: exits program with error level 0
 * DESCRIPTION   : print program usage information
 ****************************************************************************/
int usage()
{
  std::cerr << std::endl <<
    "usage: " << program_name << "\n"
	    << std::endl;
  exit(1);
}

/****************************************************************************
 * FUNCTION_NAME : print_header()
 * OUTPUT        : execution header to the output stream.
 * DESCRIPTION   : print the output header with the current date and time.
 ****************************************************************************/
void print_header()
{
  char time_str[255];
  struct tm *time_ptr;
  time_t current_time;

  // print header with current date.
  printf("\n");
  printf("%% /*---------------------------------------------------*/\n");
  printf("%%    file: pas2ps\n");
  printf("%%    usage: pas2ps [pascalDfilename] \n");
  printf("%%    pc2ps -- pascal to postscript\n");
  printf("%%    CS 541 University of Kentucky\n");
  printf("%%    Dulith Herath\n");
  printf("%%    Limitations:\n");
  printf("%%    \t see README:\n");
  // Get the Date and time...
  time(&current_time);
  time_ptr = localtime(&current_time);
  // Format the data and place it formated into time_str.
  strftime(time_str, 255, "%A, %b. %d, %Y", time_ptr);
  printf("%%    Current Date: %s\n", time_str);
  // Format the time and place it formated into time_str.
  strftime(time_str, 20, "%I:%M %p", time_ptr);
  printf("%%    Current Time: %s\n", time_str);
  printf("%% /*---------------------------------------------------*/\n");
}

/****************************************************************************
 * FUNCTION_NAME : main()
 * INPUT/OUTPUT  : main control for program execution. Each section is commented.
 * FUNCTIONS CALLED:
 *                 print_header();
 *                 print_file();
 *                 InitLex();
 *                 start_scan();
 *                 SymbolTable::FPrintF();
 *                 SymbolTable::QueryMode();
 * DESCRIPTION   :  Processes command line args.
 *                  Opens input and output streams.
 *                  Starts the scanner procedure.
 *                  Prints output.
 *                  Handles -d option with debug output print
 *                  Handles -q option by starting QueryMode.
 ****************************************************************************/
int main(int argc, char **argv)
{
  bool noheader=false;

  // Process Command line arguments.
  program_name = argv[0];
  int optIndex = 0;
  while ((optIndex = getopt(argc,argv,"hn")) != -1)
    switch (optIndex)
      {
      default: usage();
      case 'n': noheader=true; break;
      }

  if (!noheader)
    print_header();

  InitLex();
  RdParser Parser;
  Parser.rdparse();

  printf("\n%% End Program.\n");
  return 0;
}
