#include <iostream>
#include <string.h>

#include "rdtoken.h"

#include "rddefs.h"

char *TOKENS[]={
#include "adobe.h"
  "PROGRAM",
  "PROCEDURE",
  "FUNCTION",
  "BEGIN_BLOCK",
  "END_BLOCK",
  "WHILE",
  "DO",
  "IF",
  "THEN",
  "ELSE",
  "ARRAY",
  "OF",
  "VAR",
  "REAL",
  "INTEGER",
  "RELOP",
  "MULOP",
  "ADDOP",
  "SIGN",
  "MINUS",
  "PLUS",
  "MULTIPLY",
  "DIV",
  "DIVIDE",
  "MOD",
  "AND",
  "NOT",
  "OR",
  "INT_NUM",
  "REAL_NUM",
  "ID",
  "EQ",
  "GE",
  "GT",
  "LT",
  "LE",
  "NE",
  "COLON",
  "COMMA",
  "SEMICOLON",
  "ASSIGN",
  "LBRACK",
  "RBRACK",
  "LPAREN",
  "RPAREN",
  "RANGE",
  "PERIOD",
  "READ",
  "WRITE",
  "WRITELN",
  "UNKNOWN",
  "FOR",
  "TO",
  "RD_EOF",
  "RD_EMPTY",
  "END"
};
extern rdunion rdoldval,rdlval;
extern int rdlines;
int yylex(void);

RdToken::RdToken() {
  advance();
}

int RdToken::match(int t) {
  return (token==t);
}

void RdToken::advance() {
  rdoldval=rdlval;
  token=yylex();
}

void RdToken::advance(int t) {
  if(token!=t) {
    std::cerr << "Warning:" << rdlines << ": Skipping token " << TOKENS[t] << "\n";
  }
  else {
    rdoldval=rdlval;
    token=yylex();
  }
}

void RdToken::getFirst(RdTokenFuncPtr firsts, RdTokenFuncPtr follows, RdTokenFuncPtr augments) {
  while(!((this->*firsts)() || (this->*follows)() || (this->*augments)())) {
    if(token==RD_EOF) {
      break;
    }
    std::cerr << "Warning:" << rdlines << ": Skipping token " << TOKENS[token] << "\n";
    advance();
  }
}

void RdToken::getFollow(RdTokenFuncPtr follows, RdTokenFuncPtr augments) {
  while(!((this->*follows)() || (this->*augments)())) {
    if(token==RD_EOF) {
      break;
    }
    std::cerr << "Warning:" << rdlines << ": Skipping token " << TOKENS[token] << "\n";
    advance();
  }
}

int RdToken::inFirst_PROGRAM_ID_SEMICOLON_Block_PERIOD () {
  switch(token) {
  case PROGRAM:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_AllDeclaration_CompoundStatement () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case BEGIN_BLOCK:
  case VAR:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Declarations_SubprogramDeclarations () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case VAR:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_VAR_IdentifierList_COLON_Type_SEMICOLON_Declarations () {
  switch(token) {
  case VAR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_RD95EMPTY () {
  switch(token) {
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ID_IdListTail () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_COMMA_ID_IdListTail () {
  switch(token) {
  case COMMA:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_StandardType () {
  switch(token) {
  case REAL:
  case INTEGER:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ARRAY_LBRACK_INT95NUM_RANGE_INT95NUM_RBRACK_OF_StandardType () {
  switch(token) {
  case ARRAY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_INTEGER () {
  switch(token) {
  case INTEGER:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_REAL () {
  switch(token) {
  case REAL:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SubprogramDeclaration_SEMICOLON_SubprogramDeclarations () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SubprogramHead_AllDeclaration_CompoundStatement () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_FUNCTION_ID_Arguments_COLON_StandardType_SEMICOLON () {
  switch(token) {
  case FUNCTION:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_PROCEDURE_ID_Arguments_SEMICOLON () {
  switch(token) {
  case PROCEDURE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_LPAREN_ParameterList_RPAREN () {
  switch(token) {
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_IdentifierList_COLON_Type_ParameterTail () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SEMICOLON_IdentifierList_COLON_Type_ParameterTail () {
  switch(token) {
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_BEGIN95BLOCK_OptionalStatement_END95BLOCK () {
  switch(token) {
  case BEGIN_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_StatementList () {
  switch(token) {
  case BEGIN_BLOCK:
  case WHILE:
  case IF:
  case ID:
  case READ:
  case WRITE:
  case WRITELN:
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Statement_StatementListTail () {
  switch(token) {
  case BEGIN_BLOCK:
  case WHILE:
  case IF:
  case ID:
  case READ:
  case WRITE:
  case WRITELN:
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SEMICOLON_Statement_StatementListTail () {
  switch(token) {
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ID_StatementOp () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_IfStatement () {
  switch(token) {
  case IF:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_CompoundStatement () {
  switch(token) {
  case BEGIN_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_WhileStatement () {
  switch(token) {
  case WHILE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ForStatement () {
  switch(token) {
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ReadStatement () {
  switch(token) {
  case READ:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_WriteStatement () {
  switch(token) {
  case WRITE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_WriteLnStatement () {
  switch(token) {
  case WRITELN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Index_OptionalAssign () {
  switch(token) {
  case LBRACK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ProcTail_OptionalAssign () {
  switch(token) {
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_AssignStatement () {
  switch(token) {
  case ASSIGN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ASSIGN_Expression () {
  switch(token) {
  case ASSIGN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_WRITE_LPAREN_Expression_RPAREN () {
  switch(token) {
  case WRITE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_WRITELN_LPAREN_Expression_RPAREN () {
  switch(token) {
  case WRITELN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_READ_LPAREN_IdRList_RPAREN () {
  switch(token) {
  case READ:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_IdAccess_IdRTail () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_COMMA_IdAccess_IdRTail () {
  switch(token) {
  case COMMA:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_LBRACK_IndexType_RBRACK () {
  switch(token) {
  case LBRACK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_INT95NUM () {
  switch(token) {
  case INT_NUM:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ID () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ID_IdAccessTail () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Index () {
  switch(token) {
  case LBRACK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_WHILE_Expression_DO_Statement () {
  switch(token) {
  case WHILE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_FOR_ID_AssignStatement_TO_Expression_DO_Statement () {
  switch(token) {
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_IF_Expression_THEN_Statement_Testelse () {
  switch(token) {
  case IF:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ELSE_Statement () {
  switch(token) {
  case ELSE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SEMICOLON () {
  switch(token) {
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_LPAREN_ProcExpressionList_RPAREN () {
  switch(token) {
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Expression_ProcExpressionListTail () {
  switch(token) {
  case SIGN:
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case COMMA:
  case LPAREN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_COMMA_Expression_ProcExpressionListTail () {
  switch(token) {
  case COMMA:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SimpleExpression_ExpressionTail () {
  switch(token) {
  case SIGN:
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_RELOP_SimpleExpression () {
  switch(token) {
  case RELOP:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SIGN_Term_SimpleExpressionTail () {
  switch(token) {
  case SIGN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Term_SimpleExpressionTail () {
  switch(token) {
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ADDOP_Term_SimpleExpressionTail () {
  switch(token) {
  case ADDOP:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_Factor_TermTail () {
  switch(token) {
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_SIGN () {
  switch(token) {
  case SIGN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_MULOP_Sign_Factor_TermTail () {
  switch(token) {
  case MULOP:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ID_FactorOp () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_NOT_Factor () {
  switch(token) {
  case NOT:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_REAL95NUM () {
  switch(token) {
  case REAL_NUM:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_LPAREN_Expression_RPAREN () {
  switch(token) {
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirst_ProcTail () {
  switch(token) {
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstProgram () {
  switch(token) {
  case PROGRAM:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstBlock () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case BEGIN_BLOCK:
  case VAR:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstAllDeclaration () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case VAR:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstCompoundStatement () {
  switch(token) {
  case BEGIN_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstDeclarations () {
  switch(token) {
  case VAR:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstSubprogramDeclarations () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIdentifierList () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstType () {
  switch(token) {
  case ARRAY:
  case REAL:
  case INTEGER:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIdListTail () {
  switch(token) {
  case COMMA:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstStandardType () {
  switch(token) {
  case REAL:
  case INTEGER:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstSubprogramDeclaration () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstSubprogramHead () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstArguments () {
  switch(token) {
  case LPAREN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstParameterList () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstParameterTail () {
  switch(token) {
  case SEMICOLON:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstOptionalStatement () {
  switch(token) {
  case BEGIN_BLOCK:
  case WHILE:
  case IF:
  case ID:
  case READ:
  case WRITE:
  case WRITELN:
  case FOR:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstStatementList () {
  switch(token) {
  case BEGIN_BLOCK:
  case WHILE:
  case IF:
  case ID:
  case READ:
  case WRITE:
  case WRITELN:
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstStatement () {
  switch(token) {
  case BEGIN_BLOCK:
  case WHILE:
  case IF:
  case ID:
  case READ:
  case WRITE:
  case WRITELN:
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstStatementListTail () {
  switch(token) {
  case SEMICOLON:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstStatementOp () {
  switch(token) {
  case ASSIGN:
  case LBRACK:
  case LPAREN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIfStatement () {
  switch(token) {
  case IF:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstWhileStatement () {
  switch(token) {
  case WHILE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstForStatement () {
  switch(token) {
  case FOR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstReadStatement () {
  switch(token) {
  case READ:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstWriteStatement () {
  switch(token) {
  case WRITE:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstWriteLnStatement () {
  switch(token) {
  case WRITELN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIndex () {
  switch(token) {
  case LBRACK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstOptionalAssign () {
  switch(token) {
  case ASSIGN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstProcTail () {
  switch(token) {
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstAssignStatement () {
  switch(token) {
  case ASSIGN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstExpression () {
  switch(token) {
  case SIGN:
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIdRList () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIdAccess () {
  switch(token) {
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIdRTail () {
  switch(token) {
  case COMMA:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIndexType () {
  switch(token) {
  case INT_NUM:
  case ID:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstIdAccessTail () {
  switch(token) {
  case LBRACK:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstTestelse () {
  switch(token) {
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstProcExpressionList () {
  switch(token) {
  case SIGN:
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case COMMA:
  case LPAREN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstProcExpressionListTail () {
  switch(token) {
  case COMMA:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstSimpleExpression () {
  switch(token) {
  case SIGN:
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstExpressionTail () {
  switch(token) {
  case RELOP:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstTerm () {
  switch(token) {
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstSimpleExpressionTail () {
  switch(token) {
  case ADDOP:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstFactor () {
  switch(token) {
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstTermTail () {
  switch(token) {
  case MULOP:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstSign () {
  switch(token) {
  case SIGN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFirstFactorOp () {
  switch(token) {
  case LBRACK:
  case LPAREN:
  case RD_EMPTY:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowProgram () {
  switch(token) {
  case RD_EOF:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowBlock () {
  switch(token) {
  case PERIOD:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowAllDeclaration () {
  switch(token) {
  case BEGIN_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowCompoundStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:
  case PERIOD:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowDeclarations () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case BEGIN_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowSubprogramDeclarations () {
  switch(token) {
  case BEGIN_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIdentifierList () {
  switch(token) {
  case COLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowType () {
  switch(token) {
  case SEMICOLON:
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIdListTail () {
  switch(token) {
  case COLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowStandardType () {
  switch(token) {
  case SEMICOLON:
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowSubprogramDeclaration () {
  switch(token) {
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowSubprogramHead () {
  switch(token) {
  case PROCEDURE:
  case FUNCTION:
  case BEGIN_BLOCK:
  case VAR:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowArguments () {
  switch(token) {
  case COLON:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowParameterList () {
  switch(token) {
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowParameterTail () {
  switch(token) {
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowOptionalStatement () {
  switch(token) {
  case END_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowStatementList () {
  switch(token) {
  case END_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowStatementListTail () {
  switch(token) {
  case END_BLOCK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowStatementOp () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIfStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowWhileStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowForStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowReadStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowWriteStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowWriteLnStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIndex () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case MULOP:
  case ADDOP:
  case COMMA:
  case SEMICOLON:
  case ASSIGN:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowOptionalAssign () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowProcTail () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case MULOP:
  case ADDOP:
  case COMMA:
  case SEMICOLON:
  case ASSIGN:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowAssignStatement () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowExpression () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIdRList () {
  switch(token) {
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIdAccess () {
  switch(token) {
  case COMMA:
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIdRTail () {
  switch(token) {
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIndexType () {
  switch(token) {
  case RBRACK:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowIdAccessTail () {
  switch(token) {
  case COMMA:
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowTestelse () {
  switch(token) {
  case END_BLOCK:
  case ELSE:
  case SEMICOLON:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowProcExpressionList () {
  switch(token) {
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowProcExpressionListTail () {
  switch(token) {
  case RPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowSimpleExpression () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowExpressionTail () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowTerm () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case ADDOP:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowSimpleExpressionTail () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowFactor () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case MULOP:
  case ADDOP:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowTermTail () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case ADDOP:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowSign () {
  switch(token) {
  case NOT:
  case INT_NUM:
  case REAL_NUM:
  case ID:
  case LPAREN:		return 1;
  default:	return 0;
  }
}

int RdToken::inFollowFactorOp () {
  switch(token) {
  case END_BLOCK:
  case DO:
  case THEN:
  case ELSE:
  case RELOP:
  case MULOP:
  case ADDOP:
  case COMMA:
  case SEMICOLON:
  case RPAREN:
  case TO:		return 1;
  default:	return 0;
  }
}

int RdToken::inAugmentProgram () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentBlock () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentAllDeclaration () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentCompoundStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentDeclarations () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentSubprogramDeclarations () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIdentifierList () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentType () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIdListTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentStandardType () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentSubprogramDeclaration () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentSubprogramHead () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentArguments () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentParameterList () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentParameterTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentOptionalStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentStatementList () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentStatementListTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentStatementOp () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIfStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentWhileStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentForStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentReadStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentWriteStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentWriteLnStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIndex () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentOptionalAssign () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentProcTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentAssignStatement () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentExpression () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIdRList () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIdAccess () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIdRTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIndexType () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentIdAccessTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentTestelse () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentProcExpressionList () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentProcExpressionListTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentSimpleExpression () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentExpressionTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentTerm () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentSimpleExpressionTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentFactor () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentTermTail () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentSign () {
  switch(token) {		default:	return 0;
  }
}

int RdToken::inAugmentFactorOp () {
  switch(token) {		default:	return 0;
  }
}

