/****************************************************************************
* FILE : st_stack.cc
* INPUT/OUTPUT  : rd++ input file (partial)
* PROJECT: CS 541 CS-University of Kentucky
* SOURCE:   Provided as a handout by Jerzy W. Jaromczyk (based on projects)
* COPYRIGHT: University of Kentucky @1999
****************************************************************************/

/* fixed for modern compilers by Dirk Jagdmann <doj@cubic.org> */

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include "st_stack.h"

extern int compile_errors;

BOOL CompareBase(LL_Node *formal, LL_Node *actual);
BOOL CompareArray(LL_Node *formal, LL_Node *actual);
BOOL CompareRecord(LL_Node *formal, LL_Node *actual);

//extern int rdlines;
/* S_Node constructor initializes pointers for the S_Node class*/
S_Node::S_Node()
{
  symbol = NULL;
  next = NULL;
}

/* S_Node destructor */
S_Node::~S_Node()
{
  delete symbol;
}

/* L_List constructor initializes pointers and size. */
S_Stack::S_Stack()
{
    current_pos = head = last = NULL;
    pos_num = size = 0;
}

/* Stack destuctor deletes pointers to nodes */
S_Stack::~S_Stack()
{
  //Clear();
}

/* Link list deletion function, frees memory. */
void S_Stack::Clear()
{
	S_Node *prev;

	while(head != NULL){
		prev = head;
		head = head->Next();
		delete prev;
	}
	current_pos = head = NULL;
	pos_num = size = 0;
}

/* Make current_pos the last node in the list */
void S_Stack::First()
{
  current_pos = head;
  pos_num = 0;
}

/* Make current_pos the last node in the list */
void S_Stack::Last()
{
  current_pos = last;
  if(last != NULL) {
    pos_num = (size - 1);
  } else pos_num = 0;
}

/* Make current_pos the node following the current node. */
void S_Stack::Next()
{
   if(current_pos != NULL) {
	current_pos = current_pos->Next();
	pos_num++;
   }
}

BOOL CompareBase(LL_Node *formal, LL_Node *actual)
{
  if((formal)&&(actual)) {
   if(strcmp(formal->GetBaseName(), actual->GetBaseName()) != 0) {
     printf("Error:%i: Formal parameter mismatch from line %i\n",rdlines,formal->GetLineNumber());
     compile_errors++;
     return FALSE;
   }
   return TRUE;
  }
  return FALSE;
}

BOOL CompareArray(LL_Node *formal, LL_Node *actual)
{
  LL_Node *formal_list, *actual_list;
  /* Should both have ARRAY as base type. */
  if((CompareBase(formal, actual) == FALSE)) return FALSE;
  /* See if they have the same dimensions */
  if(formal->GetDimension() != actual->GetDimension()) {
    printf("Error:%i: Formal parameter: wrong dimensions line %i\n",rdlines,formal->GetLineNumber());
    compile_errors++;
    return FALSE;
  }
  formal_list = (LL_Node *)formal->GetSymbolList();
  actual_list = (LL_Node *)actual->GetSymbolList();

  if(((formal_list == NULL)&&(actual_list != NULL))||((formal_list != NULL)&&(actual_list == NULL))) {
    printf("Error:%i: Formal parameter: array types differ line %i\n",rdlines,formal->GetLineNumber());
    compile_errors++;
    return FALSE;
  }
  if(formal_list == NULL) {
    if(actual_list == NULL)
   //   printf("    acutal->GetSymbolList(): NULL\n");
    return FALSE;
  }
  if(actual_list == NULL) {
   // printf("      acutal->GetSymbolList(): NULL\n");
    return FALSE;
  }

  if(formal_list->GetDefinitionType() != actual_list->GetDefinitionType()) {
    printf("Error:%i: Formal param. def. mismatch from line %i\n",rdlines,formal->GetLineNumber());
    compile_errors++;
    return FALSE;
  }
  switch(actual_list->GetDefinitionType()) {
     case T_VAR:
       return CompareBase(formal_list, actual_list);
       break;
     case T_ARRAY:
       return CompareArray(formal_list, actual_list);
       break;
     case T_RECORD:
       return CompareRecord(formal_list, actual_list);
       break;
     default:
       if(strcmp(formal_list->GetSpelling(), actual_list->GetSpelling()) != 0) {
	 printf("Error:%i: Formal parameter mismatch from line %i\n",rdlines,formal->GetLineNumber());
	 compile_errors++;
       }
       return TRUE;
       break;
  }
  return TRUE;
}

BOOL CompareRecord(LL_Node *formal, LL_Node *actual)
{
  S_Stack *formal_fields, *actual_fields;

  if(CompareBase(formal, actual) == FALSE) return FALSE;

  formal_fields = (S_Stack *)formal->GetSymbolList();
  actual_fields = (S_Stack *)actual->GetSymbolList();

  /* Now only need to compare the two stacks above. :) */
  /* I wonder if I use the Compare member function. */

  return formal_fields->Compare(actual_fields);

}

/* Compare all the symbols of two symbol stacks, popping all
   nodes from the passed stack. */
BOOL S_Stack::Compare(S_Stack *stack)
{
  LL_Node *actual;
  LL_Node *formal;
  BOOL ret_val;

  if(this == NULL) return FALSE;
  if(stack == NULL) return FALSE;
  if(stack->GetSize() < size) {
    printf( "Error:%i: Too few parameters.\n",rdlines);
    compile_errors++;
  } else if(stack->GetSize() > size) {
    printf( "Error:%i: Too many parameters.\n",rdlines);
    compile_errors++;
  }
  First();
  stack->First();
  while((stack->GetCurrent() != NULL)&&(current_pos != NULL)) {
    formal = current_pos->GetSymbol();
    actual = stack->GetCurrent()->GetSymbol();
    if((formal == NULL)||(actual == NULL)) {
       printf("Error:%i: Undefined parameter.\n",rdlines);
       compile_errors++;
       return FALSE;
    }
    /* Step through the stack while comparing
       the values and content of each node. */
    if(actual->GetDefinitionType() != formal->GetDefinitionType()) {
      printf("Error:%i: Formal param. def. mismatch from line %i\n",rdlines,formal->GetLineNumber());
      compile_errors++;
    }
    switch(actual->GetDefinitionType()) {
     case T_VAR:
       ret_val = CompareBase(formal, actual);
       break;
     case T_ARRAY:
       ret_val = CompareArray(formal, actual);
       break;
     case T_RECORD:
       ret_val = CompareRecord(formal, actual);
       break;
     default:
        printf( "Error:%i: Unknown definition type.\n",rdlines);
	compile_errors++;
	break;
    }
    stack->Next();
    Next();
  }

  return ret_val;
}

/* Insert a symbol at the head of a list.*/
void S_Stack::Push(LL_Node *new_symbol)
{
   S_Node *new_node;

   if(new_symbol != NULL) {
     /*fprintf(stderr, "Push: %i, %s\n", rdlines, new_symbol->GetSpelling());*/
   } else {
     /*fprintf(stderr, "Push: %i, %s\n", rdlines, "NULL");*/
   }
   new_node = new S_Node;
   new_node->SetSymbol(new_symbol);
   if(head == NULL) {
     head = new_node;
     head->SetNext(NULL);
     current_pos = last = new_node;
     pos_num = 0;
     size = 1;
   } else {
     new_node->SetNext(head);
     head = current_pos = new_node;
     pos_num = size;
     size++;
   }

}

/*  "Pops" the topmost Symbol off the stack */
LL_Node *S_Stack::Pop( )
{
  S_Node *p;

  /*fprintf(stderr, "Pop: %i, ", rdlines);*/
  if(head != NULL) {
    p = head;
    head = head->Next();
    /* AS IS //delete p; don't want to do this for these guys. */
    if(p->GetSymbol() != NULL) {
      /*fprintf(stderr, "%s\n", p->GetSymbol()->GetSpelling()); */
    } else {
      /*fprintf(stderr, "NULL\n");*/
    }
    size--;
    return p->GetSymbol();
  }
  /*fprintf(stderr, "\n");*/
  return NULL;
}


/******************************ST_Stack***************************/


/* ST_Node constructor initializes pointers for the ST_Node class*/
ST_Node::ST_Node()
{
  symbol_table = NULL;
  next = NULL;
}

/* ST_Node destructor */
ST_Node::~ST_Node()
{
  delete symbol_table;
}

/* L_List constructor initializes pointers and size. */
ST_Stack::ST_Stack()
{
    current_pos = head = last = NULL;
    pos_num = size = 0;
}

/* Stack destuctor deletes pointers to nodes */
ST_Stack::~ST_Stack()
{
  //Clear();
}

/* Link list deletion function, frees memory. */
void ST_Stack::Clear()
{
	ST_Node *prev;

	while(head != NULL){
		prev = head;
		head = head->GetNext();
		delete prev;
	}
	current_pos = head = NULL;
	pos_num = size = 0;
}

/* Make current_pos the last node in the list */
void ST_Stack::First()
{
  current_pos = head;
  pos_num = 0;
}

/* Make current_pos the last node in the list */
void ST_Stack::Last()
{
  current_pos = last;
  if(last != NULL) {
    pos_num = (size - 1);
  } else pos_num = 0;
}

/* Make current_pos the node following the current node. */
void ST_Stack::Next()
{
   if(current_pos != NULL) {
	current_pos = current_pos->GetNext();
	pos_num++;
   }
}

/* Insert a symbol at the head of a list.*/
void ST_Stack::Push(Symbol_Table *new_table)
{
   ST_Node *new_node;

   fprintf(stderr, "ST Push: %i\n", rdlines);
   new_node = new ST_Node;
   new_node->SetSymbol_Table(new_table);
   if(head == NULL) {
     new_node->SetNext(NULL);
     head = new_node;
     current_pos = last = new_node;
     pos_num = 0;
     size = 1;
   } else {
     new_node->SetNext(head);
     head = current_pos = new_node;
     pos_num = size;
     size++;
   }

}

/*  "Pops" the topmost SymbolTable off the stack */
Symbol_Table *ST_Stack::Pop( )
{
  //ST_Node *p;

  /* Pop. */
  fprintf(stderr, "ST Pop: %i\n", rdlines);
  if(head != NULL) {
    head = head->GetNext();
    size--;
    /* AS IS //delete p; */
    if(head != NULL) {
      return head->GetSymbol_Table();
    }
  }
  return NULL;
}

/* Search each symbol in the symbol table stack for a
   given symbol spelling until levels symbol tables have been
   search or the a symbol with the corresponding spelling is found. */
LL_Node *ST_Stack::BackSearch(char *spelling, int levels)
{
  ST_Node *p = head;
  Symbol_Table *st_ptr;
  LL_Node *symbol_found;

  while((levels > 0)&&(p != NULL)) {
     st_ptr = p->GetSymbol_Table();
     if(st_ptr != NULL) {
        if((symbol_found = st_ptr->Lookup(spelling, -1)) != NULL) {
           /* Found the spelling. */
           return symbol_found;
        }
     }
     p = p->GetNext();
     levels--;
  }

  /* Didn't find it. */
  return NULL;
}





