/*
  st_stack.h
*/

#ifndef _ST_Stack_H_
#define _ST_Stack_H_

#include "sym_table.h"

class S_Node
{
   protected:
        LL_Node *symbol;
        S_Node *next;
   public:
	S_Node();
        ~S_Node();
        void SetSymbol(LL_Node *new_symbol) { symbol = new_symbol; }
        LL_Node *GetSymbol(void) { return symbol; }
        void SetNext(S_Node *new_next) { next = new_next; };
        S_Node *Next(void) { return next; }
};


/* A list/stack class. */
/* Nodes are inserted at the head of the list using push. */
/* Nodes can be deleted from the head of the list with pop. */
class S_Stack
{
   protected:
	S_Node *head;
        S_Node *last;
        S_Node *current_pos;
        int size, pos_num;
   public:
	S_Stack();
	~S_Stack();
  /* List Operations */
        /* Delete all nodes. */
        void Clear(void);
        /* Make current_pos the first. */
        void First(void);
        /* Make current_pos the last. */
        void Last(void);
        /* Advance current_pos */
        void Next(void);
        S_Node *GetCurrent() { return current_pos; }
        int GetSize(void) { return size; }
	int GetPosNum(void) { return pos_num; }

     /* Stack Operations */
        void Push(LL_Node *);
        LL_Node *Pop(void);

     /* (Stack && List) Operations */
        BOOL Compare(S_Stack *stack);

};

/**************************ST_Stack*******************************/

class ST_Node
{
   protected:
        Symbol_Table *symbol_table;
        ST_Node *next;
   public:
	ST_Node();
        ~ST_Node();
        void SetSymbol_Table(Symbol_Table *new_table) { symbol_table = new_table; }
        Symbol_Table *GetSymbol_Table(void) { return symbol_table; }
        void SetNext(ST_Node *new_next) { next = new_next; };
        ST_Node *GetNext(void) { return next; }

};


/* A list/stack class. */
/* Nodes are inserted at the head of the list using push. */
/* Nodes can be deleted from the head of the list with pop. */
class ST_Stack
{
   protected:
	ST_Node *head;
        ST_Node *last;
        ST_Node *current_pos;
        int size, pos_num;
   public:
	ST_Stack();
	~ST_Stack();
      /* List Operations */
        /* Delete all nodes. */
        void Clear(void);
        /* Make current_pos the first. */
        void First(void);
        /* Make current_pos the last. */
        void Last(void);
        /* Advance current_pos */
        void Next(void);
        ST_Node *GetHead() { return head; }
        ST_Node *GetCurrent() { return current_pos; }
        ST_Node *GetNext() { return current_pos->GetNext(); }
        int GetSize(void) { return size; }
	int GetPosNum(void) { return pos_num; }
        LL_Node *BackSearch(char *spelling, int levels);

      /* Stack Operations */
        void Push(Symbol_Table *);
        Symbol_Table *Pop(void);
};

#endif






